/* 
 * Sample BMF file reader
 */

#ifdef WIN32
#include "windows.h"
#endif

#include <stdio.h>
#include <stdlib.h>
#include "GL/glut.h"
#include "bmf.h"

BMFObject* LoadBMF(char *filename) {
  uword MeshType;
  SparseMesh *MaterialList;
  StripMesh  *StripMaterialList;
  BMFObject *obj;
  FILE *in;
  int i;

  /* Create the new BMF object */
  if (!(obj = (BMFObject *) malloc(sizeof(BMFObject)))) {
    printf("Can't allocate memory for BMFObject\n");
    exit(-1);
  }

  if (!(in = fopen(filename, "rb"))) {
    printf("Can't open file\n");
    exit(-1);
  }
  
  /* Get number of materials */
  fread(&obj->NumMaterials, sizeof(uword), 1, in);

  /* Get mesh type */
  fread(&MeshType, sizeof(uword), 1, in);

  if (MeshType == 0) {
    /* Sparse Triangles */

    /* Allocate space for the materials */
    if (!(MaterialList = (SparseMesh *) malloc(sizeof(SparseMesh) * obj->NumMaterials))) {
      printf("Can't allocate memory for materials\n");
      exit(-1);
    }
    obj->sparse = MaterialList;
    obj->strip  = 0;
    
    /* Read in each material */
    for (i=0; i < obj->NumMaterials; i++) {
      /* Get the texture's name length */
      fread(&MaterialList[i].texture_name_length, 
	    sizeof(uword), 
	    1, in);
      
      if (MaterialList[i].texture_name_length == 0) {
	printf("Invalid texture name length\n");
	exit(-1);
      }
      
      /* Get the texture's name */
      MaterialList[i].texture_name = 
	(byte *) malloc(MaterialList[i].texture_name_length);
      fread(MaterialList[i].texture_name, 
	    MaterialList[i].texture_name_length, 
	    1, in);
      
      /* Read in RGB colors */
      fread(&MaterialList[i].ambient, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      fread(&MaterialList[i].diffuse, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      fread(&MaterialList[i].specular, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      /* Get the number of vertices */
      fread(&MaterialList[i].number_of_vertices, 
	    sizeof(uword), 
	    1, in);
      
      /* Allocate space for the vertex list */
      MaterialList[i].vertexlist = 
	(BMF_UNVERTEX *) malloc(sizeof(BMF_UNVERTEX) * MaterialList[i].number_of_vertices);
      
      if (MaterialList[i].vertexlist == 0) {
	printf("Error allocating memory for vertexlist\n");
	exit(-1);
      }
      
      /* Read in the vertex list */
      fread(MaterialList[i].vertexlist, 
	    sizeof(BMF_UNVERTEX), 
	    MaterialList[i].number_of_vertices, 
	    in);

      /* Read in the number of polygons */
      fread(&MaterialList[i].number_of_triangles,
	    sizeof(uword),
	    1,
	    in);
      
      /* Allocate space for the index list */
      MaterialList[i].indexlist = 
	(BMF_SURFACE *)malloc(sizeof(BMF_SURFACE) * MaterialList[i].number_of_triangles);
      
      /* Read in the triangle indices */
      fread(MaterialList[i].indexlist,
	    sizeof(BMF_SURFACE),
	    MaterialList[i].number_of_triangles,
	    in);
    }
  } else {
    /* Triangle Strips */
    
    /* Allocate space for the materials */
    if (!(StripMaterialList = (StripMesh *) malloc(sizeof(StripMesh) * obj->NumMaterials))) {
      printf("Can't allocate memory for materials\n");
      exit(-1);
    }
    obj->sparse = 0;
    obj->strip  = StripMaterialList;
    
    /* Read in each material */
    for (i=0; i < obj->NumMaterials; i++) {
      /* Get the texture's name length */
      fread(&StripMaterialList[i].texture_name_length, 
	    sizeof(uword), 
	    1, in);
      
      if (StripMaterialList[i].texture_name_length == 0) {
	printf("Invalid texture name length\n");
	exit(-1);
      }
      
      /* Get the texture's name */
      StripMaterialList[i].texture_name = 
	(byte *) malloc(StripMaterialList[i].texture_name_length);
      fread(StripMaterialList[i].texture_name, 
	    StripMaterialList[i].texture_name_length, 
	    1, in);
      
      /* Read in RGB colors */
      fread(&StripMaterialList[i].ambient, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      fread(&StripMaterialList[i].diffuse, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      fread(&StripMaterialList[i].specular, 
	    sizeof(BMF_RGBCOLOR), 
	    1, in);
      
      /* Get the number of vertices */
      fread(&StripMaterialList[i].number_of_vertices, 
	    sizeof(uword), 
	    1, in);

      /* Allocate space for the vertex list */
      StripMaterialList[i].vertexlist = 
	(BMF_UNVERTEX *) malloc(sizeof(BMF_UNVERTEX) * StripMaterialList[i].number_of_vertices);
      
      if (StripMaterialList[i].vertexlist == 0) {
	printf("Error allocating memory for vertexlist\n");
	exit(-1);
      }
      
      /* Read in the vertex list */
      fread(StripMaterialList[i].vertexlist, 
	    sizeof(BMF_UNVERTEX), 
	    StripMaterialList[i].number_of_vertices, 
	    in);
      
      /* Read in the number of strips */
      fread(&StripMaterialList[i].number_of_strips,
	    sizeof(uword),
	    1,
	    in);
      
      /* Allocate space for the strip length list */
      StripMaterialList[i].length_of_strip = 
	(uword *)malloc(sizeof(uword) * StripMaterialList[i].number_of_strips);
      
      /* Read in the triangle strip lengths */
      fread(StripMaterialList[i].length_of_strip,
	    sizeof(uword),
	    StripMaterialList[i].number_of_strips,
	    in);
      
      /* Read in the number of strip points */
      fread(&StripMaterialList[i].number_of_strip_indices,
	    sizeof(uword),
	    1,
	    in);

      /* Allocate space for the strip length list */
      StripMaterialList[i].stripindex = 
	(uword *)malloc(sizeof(uword) * StripMaterialList[i].number_of_strip_indices);
      
      /* Read in the triangle strip lengths */
      fread(StripMaterialList[i].stripindex,
	    sizeof(uword),
	    StripMaterialList[i].number_of_strip_indices,
	    in);
    }
  }

  fclose(in);

  return obj;
}



void DrawBMFObject(BMFObject *obj) {
  int i,j;
  int current;

  if (obj->sparse) {
    /* Draw Sparse Triangles */
    for (i=0; i < obj->NumMaterials; i++) {
      /* Tell OpenGL where our vertices are at */
      glVertexPointer(3, GL_FLOAT, sizeof(BMF_UNVERTEX), &obj->sparse[i].vertexlist[0].x);
      glNormalPointer(GL_FLOAT, sizeof(BMF_UNVERTEX), &obj->sparse[i].vertexlist[0].i);

      /* Set up this material */
      glMaterialfv(GL_FRONT, GL_AMBIENT,  (GLfloat *) &obj->sparse[i].ambient);
      glMaterialfv(GL_FRONT, GL_DIFFUSE,  (GLfloat *) &obj->sparse[i].diffuse);
      glMaterialfv(GL_FRONT, GL_SPECULAR, (GLfloat *) &obj->sparse[i].specular);

      /* Draw the triangles */
      glDrawElements(GL_TRIANGLES,
		     obj->sparse[i].number_of_triangles*3, 
      		     GL_UNSIGNED_SHORT, 
		     obj->sparse[i].indexlist);
    }
  } else {
    /* Draw Triangle Strips */
    for (i=0; i < obj->NumMaterials; i++) {
      /* Tell OpenGL where our data is at */
      glVertexPointer(3, GL_FLOAT, sizeof(BMF_UNVERTEX), &obj->strip[i].vertexlist[0].x);
      glNormalPointer(GL_FLOAT, sizeof(BMF_UNVERTEX), &obj->strip[i].vertexlist[0].i);

      
      /* Set up this material */
      glMaterialfv(GL_FRONT, GL_AMBIENT,  (GLfloat *) &obj->strip[i].ambient);
      glMaterialfv(GL_FRONT, GL_DIFFUSE,  (GLfloat *) &obj->strip[i].diffuse);
      glMaterialfv(GL_FRONT, GL_SPECULAR, (GLfloat *) &obj->strip[i].specular);

      /* Draw each strip at a time for this material */
      current = 0;
      for (j=0; j<obj->strip[i].number_of_strips; j++) {
	glDrawElements(GL_TRIANGLE_STRIP, 
		       obj->strip[i].length_of_strip[j], 
		       GL_UNSIGNED_SHORT, 
		       &obj->strip[i].stripindex[current]);
	current += obj->strip[i].length_of_strip[j];
      }
    }
  }
}