#include "camera.h"


/**********
* Rotate camera around x-axis of scene's center.
*
* angle
*	Number of degrees to rotate. Positive for counterclockwise camera
*	rotation (i.e. camera nose up), negative for clockwise camera
*	rotation (i.e. camera nose down).
**********/
void camera_t::center_pitch(GLfloat angle)
{
//	move_forward(separation);
//	camera_t::change_pitch(angle);
//	move_back(separation);
//	glGetFloatv(GL_MODELVIEW_MATRIX, transformation);
}


/**********
* Rotate camera around y-axis of scene's center.
*
* angle
*	Number of degrees to rotate. Positive for counterclockwise camera
*	rotation, negative for clockwise camera rotation.
**********/
void camera_t::center_yaw(GLfloat angle)
{
//	move_forward(separation);
//	camera_t::change_yaw(angle);
//	move_back(separation);
//	glGetFloatv(GL_MODELVIEW_MATRIX, transformation);
}


/**********
* Retrieve requested camera attributes. Requesting an unknown attribute
* is undefined.
*
* which_attr
*	Name of attribute to retrieve.
*
* attr
*	The requested attribute. The attribute specified by which_attr is
*	expected to be a vector or a set of coordinates, so attr must be a
*	3-element array to contain the returned attribute.
**********/
void camera_t::get_attr_fv(GLenum which_attr, GLfloat *attr)
{
//	GLfloat m[16];

	if (which_attr == OBJ_FRONT) {
/*		invert_matrix(transformation, m);
		attr[0] = -m[8];
		attr[1] = -m[9];
		attr[2] = -m[10];*/
		attr[0] = -transformation[8];
		attr[1] = -transformation[9];
		attr[2] = -transformation[10];
	}
	else
		world_object_t::get_attr_fv(which_attr, attr);
}


/**********
* Render the camera, ignoring position and orientation. I.e. draw the
* camera wherever the current (MODELVIEW) matrix dictates.
*
* It doesn't make much sense to draw an enabled camera, since your
* viewing the world from that camera! But it's perfectly legal if you
* really want to waste your time doing so. :)
**********/
void camera_t::draw_here()
{
	GLfloat cube_length = 0.5f;
	GLfloat cone_radius = 0.25f;
	GLfloat cone_height = 0.15f;
	GLfloat small_cube_length = 0.1f;

	glutWireCube(cube_length);
	glPushMatrix();
	glTranslatef(0,0,-cube_length/2-(cone_height));
	glutWireCone(cone_radius, cone_height, 10, 10);
	glPopMatrix();
	glPushMatrix();
	glTranslatef(0,cube_length/2+small_cube_length/2,0);
	glutWireCube(small_cube_length);
	glPopMatrix();
}


/**********
* Enable the camera (so that the view of the 3D world will now follow
* this camera).
*
* Only one camera should be enabled at any given time per window.
**********/
void camera_t::enable()
{
	enabled = 1;
	refresh();
	glutPostRedisplay();
}


/**********
* Disable the camera (so that the view of the 3D world will no longer
* follow this camera).
*
* Only one camera should be enabled at any given time.
**********/
void camera_t::disable()
{
	enabled = 0;
}


/**********
* Return whether the camera is enabled.
**********/
GLint camera_t::is_enabled()
{
	return enabled;
}


/**********
* Refresh the current matrix with camera's settings.
*
* The current matrix (MODELVIEW) is set according to camera placement
* and orientation.
*
* refresh() should only be called for an enabled camera.
**********/
void camera_t::refresh()
{
	GLfloat m[16];

	invert_matrix(transformation, m);
	glLoadMatrixf(m);
}


/**********
* Initializes camera with specified attributes.
**********/
camera_t::camera_t(GLfloat init_pos_x,
		GLfloat init_pos_y,
		GLfloat init_pos_z,
		GLfloat lookat_x,
		GLfloat lookat_y,
		GLfloat lookat_z,
		GLfloat up_x,
		GLfloat up_y,
		GLfloat up_z) :
				world_object_t(NULL, init_pos_x, init_pos_y, init_pos_z,
				lookat_x, lookat_y, lookat_z, up_x, up_y, up_z)
{
	enabled = 0;
}