/* Simulate the effects of a camera.
 *
 * Affects current modelview matrix, it is expected that MODELVIEW be
 * current when calling any member functions.
 *
 * GL rendering context must be initialized prior to creating an
 * instance of this object.
 *
 * Remember to enable a camera for your 3D world, otherwise the
 * MODELVIEW matrix won't be affected by a camera. The view of the 3D
 * world will follow an enabled camera. Once a camera is disabled, the
 * view will stop following that camera. Don't have more than one
 * camera enabled at any given time per window.
 */

#ifndef _CAMERA_H
#define _CAMERA_H


#include <GL/glut.h>
#include "more_math.h"
#include "world_object.h"

class camera_t:public world_object_t {
public:
	void move_back() { world_object_t::move_back(); if (enabled) refresh(); }
	void move_forward() { world_object_t::move_forward(); if (enabled) refresh(); }
	void move_right() { world_object_t::move_right(); if (enabled) refresh(); }
	void move_left() { world_object_t::move_left(); if (enabled) refresh(); }
	void move_up() { world_object_t::move_up(); if (enabled) refresh(); }
	void move_down() { world_object_t::move_down(); if (enabled) refresh(); }
	void pitch_back(GLfloat angle) { world_object_t::pitch_back(angle); if (enabled) refresh(); }
	void pitch_back() { world_object_t::pitch_back(); if (enabled) refresh(); }
	void pitch_forward(GLfloat angle) { world_object_t::pitch_forward(angle); if (enabled) refresh(); }
	void pitch_forward() { world_object_t::pitch_forward(); if (enabled) refresh(); }
	void yaw_left(GLfloat angle) { world_object_t::yaw_left(angle); if (enabled) refresh(); }
	void yaw_left() { world_object_t::yaw_left(); if (enabled) refresh(); }
	void yaw_right(GLfloat angle) { world_object_t::yaw_right(angle); if (enabled) refresh(); }
	void yaw_right() { world_object_t::yaw_right(); if (enabled) refresh(); }
	void roll_left(GLfloat angle) { world_object_t::roll_left(angle); if (enabled) refresh(); }
	void roll_left() { world_object_t::roll_left(); if (enabled) refresh(); }
	void roll_right(GLfloat angle) { world_object_t::roll_right(angle); if (enabled) refresh(); }
	void roll_right() { world_object_t::roll_right(); if (enabled) refresh(); }
/**/	void center_pitch(GLfloat angle);
/**/	void center_yaw(GLfloat angle);
	void center_roll_left() { roll_left(); }
	void center_roll_right() { roll_right(); }
	void get_attr_fv(GLenum which_attr, GLfloat *attr);
	void enable();
	void disable();
	GLint is_enabled();
	void refresh();
	camera_t(GLfloat init_pos_x = 0.0f,
			GLfloat init_pos_y = 0.0f,
			GLfloat init_pos_z = 10.0f,
			GLfloat lookat_x = 0.0f,
			GLfloat lookat_y = 0.0f,
			GLfloat lookat_z = 0.0f,
			GLfloat up_x = 0.0f,
			GLfloat up_y = 1.0f,
			GLfloat up_z = 0.0f);

private:
	void draw_here();
	GLint enabled;	// When a camera is enabled, the MODELVIEW matrix
					// will be updated every time the camera changes
					// position or orientation. Only one camera should
					// be enabled at any given time.
};


#endif