/* Windows specific code is used, namely GetCursorPos(), SetCursorPos(),
 * POINT, GetAsyncKeyState().
 */


#include "world.h"


/**********
* Globals
**********/
long mouse_pos[2] = {-1, -1};	// Saved mouse position.


/**********
* mouse_motion()
*
* Process mouse motion input that affects the world.
**********/
void world_t::mouse_motion()
{
	long new_pos[2];	// New mouse position.
	int left_edge, right_edge, top_edge, bottom_edge;

	GetCursorPos( (POINT *)new_pos );

	if (new_pos[0] == mouse_pos[0] && new_pos[1] == mouse_pos[1])
		return;

	left_edge = glutGet(GLUT_WINDOW_X) + MOUSE_MARGIN;
	right_edge = glutGet(GLUT_WINDOW_X) + glutGet(GLUT_WINDOW_WIDTH) - MOUSE_MARGIN;
	top_edge = glutGet(GLUT_WINDOW_Y) + MOUSE_MARGIN;
	bottom_edge = glutGet(GLUT_WINDOW_Y) + glutGet(GLUT_WINDOW_HEIGHT) - MOUSE_MARGIN;

	// Wrap mouse around left/right edge.
	if (new_pos[0] < left_edge) {
		new_pos[0] = right_edge - (left_edge - new_pos[0]);
		mouse_pos[0] = right_edge;
		SetCursorPos(new_pos[0], new_pos[1]);
	}
	else if (new_pos[0] > right_edge) {
		new_pos[0] = left_edge + (new_pos[0] - right_edge);
		mouse_pos[0] = left_edge;
		SetCursorPos(new_pos[0], new_pos[1]);
	}

	// Wrap mouse around top/bottom edge.
	if (new_pos[1] < top_edge) {
		new_pos[1] = bottom_edge - (top_edge - new_pos[1]);
		mouse_pos[1] = bottom_edge;
		SetCursorPos(new_pos[0], new_pos[1]);
	}
	else if (new_pos[1] > bottom_edge) {
		new_pos[1] = top_edge + (new_pos[1] - bottom_edge);
		mouse_pos[1] = top_edge;
		SetCursorPos(new_pos[0], new_pos[1]);
	}

/*
	// Mouse wraps around at left/right window edge.
	if (new_pos[0] == 0) {
		mouse_pos[0] = glutGet(GLUT_WINDOW_WIDTH) - 1;
		SetCursorPos(mouse_pos[0]-1, new_pos[1]);
		return;
	}
	else if (new_pos[0] == glutGet(GLUT_WINDOW_WIDTH) - 1) {
		mouse_pos[0] = 0;
		SetCursorPos(1, new_pos[1]);
		return;
	}

	// Mouse wraps around at top/bottom window edge.
	if (new_pos[1] == 0) {
		mouse_pos[1] = glutGet(GLUT_WINDOW_HEIGHT) - 1;
		SetCursorPos(new_pos[0], mouse_pos[1]-1);
		return;
	}
	else if (new_pos[1] == glutGet(GLUT_WINDOW_HEIGHT) - 1) {
		mouse_pos[1] = 0;
		SetCursorPos(new_pos[0], 1);
		return;
	}
*/

	camera->yaw_left( (mouse_pos[0] - new_pos[0]) * MOUSE_SENSITIVITY );
	camera->pitch_back( (mouse_pos[1] - new_pos[1]) * MOUSE_SENSITIVITY );
//	glutPostRedisplay();
	memcpy( mouse_pos, new_pos, sizeof(new_pos) );
}


/**********
* keystrokes()
*
* Process keystroke input that affects the world.
**********/
void world_t::keystrokes()
{
	GLfloat old_speed;
//	static int in_transition;	// Flag if player is in transition from a moved state to a stationary state.

	if (GetAsyncKeyState('A') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_X);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_X, 2*old_speed);
		camera->move_left();
		camera->set_attr_f(OBJ_SPEED_X, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('D') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_X);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_X, 2*old_speed);
		camera->move_right();
		camera->set_attr_f(OBJ_SPEED_X, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('W') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_Z);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_Z, 2*old_speed);
		camera->move_forward();
		camera->set_attr_f(OBJ_SPEED_Z, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('S') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_Z);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_Z, 2*old_speed);
		camera->move_back();
		camera->set_attr_f(OBJ_SPEED_Z, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('R') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_Y);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_Y, 2*old_speed);
		camera->move_up();
		camera->set_attr_f(OBJ_SPEED_Y, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('F') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_Y);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_Y, 2*old_speed);
		camera->move_down();
		camera->set_attr_f(OBJ_SPEED_Y, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('Q') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_ROLL);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_ROLL, 2*old_speed);
		camera->roll_left();
		camera->set_attr_f(OBJ_SPEED_ROLL, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState('E') < 0) {
		old_speed = camera->get_attr_f(OBJ_SPEED_ROLL);
		if (GetAsyncKeyState(VK_SHIFT) < 0)
			camera->set_attr_f(OBJ_SPEED_ROLL, 2*old_speed);
		camera->roll_right();
		camera->set_attr_f(OBJ_SPEED_ROLL, old_speed);
//		glutPostRedisplay();
	}

	if (GetAsyncKeyState(VK_SUBTRACT) < 0)
		camera->disable();
	else if (GetAsyncKeyState(VK_ADD) < 0)
		camera->enable();
}


/**********
* process_input()
*
* Process any user input that will affect the world.
**********/
void world_t::process_input()
{
	if (is_paused)
		return;
	mouse_motion();
	keystrokes();
}


/**********
* update()
*
* Update the world. Move any moving objects, game AI, etc.
**********/
void world_t::update()
{
	if (is_paused)
		return;
}


/**********
* suspend()
*
* Suspend everything in the world. Nothing is processed until resume()
* is called. The only public member functions available while suspended
* are:
*	resume()
*	draw()
*	~world_t()
* Calling any other public member functions while suspended will have
* no effect.
**********/
void world_t::suspend()
{
	is_paused = 1;
}


/**********
* resume()
*
* Unsuspend world. If world was not already suspended, resume() has
* no effect.
**********/
void world_t::resume()
{
	is_paused = 0;
	GetCursorPos( (POINT *)mouse_pos );
}


/**********
* is_suspended()
*
* Return whether the world is suspended.
**********/
int world_t::is_suspended()
{
	return is_paused;
}


/**********
* draw()
*
* Draw all visible objects in the world.
**********/
void world_t::draw()
{
	// Draw objects.
	obj->draw();

	// Draw camera, but only if it's disabled.
	if ( !camera->is_enabled() )
		camera->draw();
}


/**********
* reset()
*
* Reset world to initial default state.
**********/
void world_t::reset()
{
	if (is_paused)
		return;

	camera->place_at(0.0,0.0,10.0, 0.0,0.0,0.0, 0.0,1.0,0.0);
	camera->enable();
	camera->set_attr_f(OBJ_SPEED_X, CAMERA_MOVE_SPEED);
	camera->set_attr_f(OBJ_SPEED_Y, CAMERA_MOVE_SPEED);
	camera->set_attr_f(OBJ_SPEED_Z, CAMERA_MOVE_SPEED);
	camera->set_attr_f(OBJ_SPEED_PITCH, CAMERA_ROTATE_SPEED);
	camera->set_attr_f(OBJ_SPEED_YAW, CAMERA_ROTATE_SPEED);
	camera->set_attr_f(OBJ_SPEED_ROLL, CAMERA_ROTATE_SPEED);

	obj->place_at(0.0,0.0,0.0, 0.0,0.0,-1.0, 0.0,1.0,0.0);
	obj->set_attr_f(OBJ_SPEED_X, DEFAULT_MOVE_SPEED);
	obj->set_attr_f(OBJ_SPEED_Y, DEFAULT_MOVE_SPEED);
	obj->set_attr_f(OBJ_SPEED_Z, DEFAULT_MOVE_SPEED);
	obj->set_attr_f(OBJ_SPEED_PITCH, DEFAULT_ROTATE_SPEED);
	obj->set_attr_f(OBJ_SPEED_YAW, DEFAULT_ROTATE_SPEED);
	obj->set_attr_f(OBJ_SPEED_ROLL, DEFAULT_ROTATE_SPEED);
}


/**********
* world_t()
*
* Constructor. Creates world objects with startup positions/orientations.
**********/
world_t::world_t()
{
	// Determine initial mouse position. Uses Windows specific code.
/*	GetCursorPos(&pt);
	mouse_pos[0] = pt.x;
	mouse_pos[1] = pt.y;*/
	GetCursorPos( (POINT *)mouse_pos );
	is_paused = 0;
	
	// Create scene objects.
	camera = new camera_t();
	obj = new world_object_t("models/graveyrd.bmf");
	reset();
}


/**********
* ~world_t()
*
* Destructor. Deletes all objects in the world.
**********/
world_t::~world_t()
{
	delete camera;
	delete obj;
}